/*----------------------------------------------------------------------------
 *      RL-ARM - RTX
 *----------------------------------------------------------------------------
 *      Name:    Blinky.c
 *      Purpose: RTX example program
 *----------------------------------------------------------------------------
 *      This code is part of the RealView Run-Time Library.
 *      Copyright (c) 2004 - 2015 KEIL - An ARM Company. All rights reserved.
 *---------------------------------------------------------------------------*/

#include "cmsis_os.h"                   // ARM::CMSIS:RTOS:Keil RTX
#include "stm32f3xx.h"                  // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_GLCD.h"                 // ::Board Support:Graphic LCD
#include "GLCD_Config.h"                // Keil.STM32373C-EVAL::Board Support:Graphic LCD


osThreadId tid_phaseA;                  /* Thread id of thread: phase_a      */
osThreadId tid_phaseB;                  /* Thread id of thread: phase_b      */
osThreadId tid_phaseC;                  /* Thread id of thread: phase_c      */
osThreadId tid_phaseD;                  /* Thread id of thread: phase_d      */
osThreadId tid_clock;                   /* Thread id of thread: clock        */
osThreadId tid_lcd;                     /* Thread id of thread: lcd          */

osMutexId mut_GLCD;                     /* Mutex to controll GLCD access     */

#define LED_A   0
#define LED_B   1
#define LED_C   2
#define LED_D   3
#define LED_CLK 7

extern GLCD_FONT     GLCD_Font_16x24;


/*----------------------------------------------------------------------------
 *      Switch LED on
 *---------------------------------------------------------------------------*/
void Switch_On (uint32_t led) {

  LED_On(led);
  osMutexWait(mut_GLCD, osWaitForever);
  GLCD_SetBackgroundColor (GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor (GLCD_COLOR_GREEN);
  GLCD_DrawChar ((5+led)*16, 4*24, 0x80+1);     /* Circle Full               */
  osMutexRelease(mut_GLCD);
}

/*----------------------------------------------------------------------------
 *      Switch LED off
 *---------------------------------------------------------------------------*/
void Switch_Off (uint32_t led) {

  LED_Off(led);
  osMutexWait(mut_GLCD, osWaitForever);
  GLCD_SetBackgroundColor (GLCD_COLOR_WHITE);
  GLCD_SetForegroundColor (GLCD_COLOR_GREEN);
  GLCD_DrawChar ((5+led)*16, 4*24, 0x80+0);     /* Circle Empty              */
  osMutexRelease(mut_GLCD);
}


/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId tid)  {
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osSignalSet(tid_clock, 0x0100);           /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osSignalSet(tid, 0x0001);                 /* set signal to thread 'thread' */
  osDelay(500);                             /* delay 500ms                   */
}

/*----------------------------------------------------------------------------
 *      Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    Switch_On (LED_A);
    signal_func(tid_phaseB);                /* call common signal function   */
    Switch_Off(LED_A);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    Switch_On (LED_B);
    signal_func(tid_phaseC);                /* call common signal function   */
    Switch_Off(LED_B);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 3 'phaseC': Phase C output
 *---------------------------------------------------------------------------*/
void phaseC (void const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    Switch_On (LED_C);
    signal_func(tid_phaseD);                /* call common signal function   */
    Switch_Off(LED_C);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 4 'phaseD': Phase D output
 *---------------------------------------------------------------------------*/
void phaseD (void  const *argument) {
  for (;;) {
    osSignalWait(0x0001, osWaitForever);    /* wait for an event flag 0x0001 */
    Switch_On (LED_D);
    signal_func(tid_phaseA);                /* call common signal function   */
    Switch_Off(LED_D);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 5 'clock': Signal Clock
 *---------------------------------------------------------------------------*/
void clock (void  const *argument) {
  for (;;) {
    osSignalWait(0x0100, osWaitForever);    /* wait for an event flag 0x0100 */
    Switch_On (LED_CLK);
    osDelay(80);                            /* delay 80ms                    */
    Switch_Off(LED_CLK);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 6 'lcd': LCD Control thread
 *---------------------------------------------------------------------------*/
void lcd (void const *argument) {

  for (;;) {
    osMutexWait(mut_GLCD, osWaitForever);
    GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
    GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
    GLCD_DrawString (0*16, 0*24, " STM32373C-EVAL Demo");
    GLCD_DrawString (0*16, 1*24, " RTX Blinky Example ");
    GLCD_DrawString (0*16, 2*24, "   www.keil.com     ");
    osMutexRelease(mut_GLCD);
    osDelay(4000);

    osMutexWait(mut_GLCD, osWaitForever);
    GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
    GLCD_SetForegroundColor (GLCD_COLOR_RED);
    GLCD_DrawString (0*16, 0*24, " STM32373C-EVAL Demo");
    GLCD_DrawString (0*16, 1*24, " RTX Blinky Example ");
    GLCD_DrawString (0*16, 2*24, "   www.keil.com     ");
    osMutexRelease(mut_GLCD);
    osDelay(4000);
  }
}


osMutexDef(mut_GLCD);

osThreadDef(phaseA, osPriorityNormal, 1, 0);
osThreadDef(phaseB, osPriorityNormal, 1, 0);
osThreadDef(phaseC, osPriorityNormal, 1, 0);
osThreadDef(phaseD, osPriorityNormal, 1, 0);
osThreadDef(clock,  osPriorityNormal, 1, 0);
osThreadDef(lcd,    osPriorityNormal, 1, 0);

/*----------------------------------------------------------------------------
  configure SystemCoreClock:
     System Clock source         PLL (HSE)
     HSE Frequency(Hz)           8000000
     SYSCLK(Hz)                  72000000
     HCLK(Hz)                    72000000
     AHB Prescaler               1
     APB2 Prescaler              1
     APB1 Prescaler              2
     PLLMUL                      9
     PREDIV                      1
     USB Clock                   DISABLE
     Flash Latency(WS)           2
     Prefetch Buffer             ON
 *----------------------------------------------------------------------------*/
static void SystemCoreClockConfigure(void) {

  RCC->CR |= ((uint32_t)RCC_CR_HSEON);               /* Enable HSE */
  while ((RCC->CR & RCC_CR_HSERDY) != RCC_CR_HSERDY) __NOP();

  FLASH->ACR = (FLASH_ACR_PRFTBE   |                 /* Enable Prefetch Buffer */
                FLASH_ACR_LATENCY_1 );               /* Set Flash Latency */

   RCC->CFGR |= (RCC_CFGR_HPRE_DIV1  |               /* HCLK = SYSCLK / 1 */
                 RCC_CFGR_PPRE2_DIV1 |               /* PCLK2 = HCLK / 1 */
                 RCC_CFGR_PPRE1_DIV2  );             /* PCLK1 = HCLK / 2 */

  RCC->CFGR &= ~(RCC_CFGR_PLLSRC   |                 /* clear settings */
                 RCC_CFGR_PLLXTPRE |
                 RCC_CFGR_PLLMUL    );
  RCC->CFGR |=  (RCC_CFGR_PLLSRC_HSE_PREDIV        | /* configure PLL */
                 RCC_CFGR_PLLXTPRE_HSE_PREDIV_DIV1 |
                 RCC_CFGR_PLLMUL9                   );


  RCC->CR |= RCC_CR_PLLON;                           /* Enable PLL */
  while((RCC->CR & RCC_CR_PLLRDY) == 0) __NOP();     /* Wait till PLL is ready */

  RCC->CFGR &= ~(RCC_CFGR_SW);                       /* clear settings */
  RCC->CFGR |= RCC_CFGR_SW_PLL;                      /* Select PLL as system clock source */
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL) __NOP();
}

/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
int main (void) {

  SystemCoreClockConfigure();
  SystemCoreClockUpdate();

  LED_Initialize();                         /* Initialize the LEDs           */
  GLCD_Initialize();                        /* Initialize the GLCD           */

  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_SetBackgroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen ();

  mut_GLCD = osMutexCreate(osMutex(mut_GLCD));

  tid_phaseA = osThreadCreate(osThread(phaseA), NULL);
  tid_phaseB = osThreadCreate(osThread(phaseB), NULL);
  tid_phaseC = osThreadCreate(osThread(phaseC), NULL);
  tid_phaseD = osThreadCreate(osThread(phaseD), NULL);
  tid_clock  = osThreadCreate(osThread(clock),  NULL);
  tid_lcd    = osThreadCreate(osThread(lcd),    NULL);

  osSignalSet(tid_phaseA, 0x0001);          /* set signal to phaseA thread   */

  osDelay(osWaitForever);
}
