/* -------------------------------------------------------------------------- 
 * Copyright (c) 2013-2016 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      Name:    BLinky.c
 *      Purpose: RTX example program
 *
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include "stm32f3xx.h"                  // Device header
#include "cmsis_os2.h"                  // ARM::CMSIS:RTOS2:Keil RTX5
#include "Board_LED.h"                  // ::Board Support:LED


osThreadId_t tid_phaseA;                /* Thread id of thread: phase_a      */
osThreadId_t tid_phaseB;                /* Thread id of thread: phase_b      */
osThreadId_t tid_phaseC;                /* Thread id of thread: phase_c      */
osThreadId_t tid_phaseD;                /* Thread id of thread: phase_d      */
osThreadId_t tid_clock;                 /* Thread id of thread: clock        */

/*----------------------------------------------------------------------------
 *      Function 'signal_func' called from multiple threads
 *---------------------------------------------------------------------------*/
void signal_func (osThreadId_t tid)  {
  osThreadFlagsSet(tid_clock, 0x0100);      /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osThreadFlagsSet(tid_clock, 0x0100);      /* set signal to clock thread    */
  osDelay(500);                             /* delay 500ms                   */
  osThreadFlagsSet(tid, 0x0001);            /* set signal to thread 'thread' */
  osDelay(500);                             /* delay 500ms                   */
}

/*----------------------------------------------------------------------------
 *      Thread 1 'phaseA': Phase A output
 *---------------------------------------------------------------------------*/
void phaseA (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny ,osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (0);
    signal_func(tid_phaseB);                                     /* call common signal function   */
    LED_Off (0);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 2 'phaseB': Phase B output
 *---------------------------------------------------------------------------*/
void phaseB (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (1);
    signal_func(tid_phaseC);                /* call common signal function   */
    LED_Off (1);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 3 'phaseC': Phase C output
 *---------------------------------------------------------------------------*/
void phaseC (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (2);
    signal_func(tid_phaseD);                /* call common signal function   */
    LED_Off (2);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 4 'phaseD': Phase D output
 *---------------------------------------------------------------------------*/
void phaseD (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0001, osFlagsWaitAny, osWaitForever);    /* wait for an event flag 0x0001 */
    LED_On (3);
    signal_func(tid_phaseA);                /* call common signal function   */
    LED_Off (3);
  }
}

/*----------------------------------------------------------------------------
 *      Thread 5 'clock': Signal Clock
 *---------------------------------------------------------------------------*/
void clock (void *argument) {
  for (;;) {
    osThreadFlagsWait(0x0100, osFlagsWaitAny, osWaitForever);    /* wait for an event flag 0x0100 */
    osDelay(80);                            /* delay  80ms                   */
  }
}

/*----------------------------------------------------------------------------
  configure SystemCoreClock:
     System Clock source         PLL (HSE)
     HSE Frequency(Hz)           8000000
     SYSCLK(Hz)                  72000000
     HCLK(Hz)                    72000000
     AHB Prescaler               1
     APB2 Prescaler              1
     APB1 Prescaler              2
     PLLMUL                      9
     PREDIV                      1
     USB Clock                   DISABLE
     Flash Latency(WS)           2
     Prefetch Buffer             ON
 *----------------------------------------------------------------------------*/
static void SystemCoreClockConfigure(void) {

  RCC->CR |= ((uint32_t)RCC_CR_HSEON);               /* Enable HSE */
  while ((RCC->CR & RCC_CR_HSERDY) != RCC_CR_HSERDY) __NOP();

  FLASH->ACR = (FLASH_ACR_PRFTBE   |                 /* Enable Prefetch Buffer */
                FLASH_ACR_LATENCY_1 );               /* Set Flash Latency */

   RCC->CFGR |= (RCC_CFGR_HPRE_DIV1  |               /* HCLK = SYSCLK / 1 */
                 RCC_CFGR_PPRE2_DIV1 |               /* PCLK2 = HCLK / 1 */
                 RCC_CFGR_PPRE1_DIV2  );             /* PCLK1 = HCLK / 2 */

  RCC->CFGR &= ~(RCC_CFGR_PLLSRC   |                 /* clear settings */
                 RCC_CFGR_PLLXTPRE |
                 RCC_CFGR_PLLMUL    );
  RCC->CFGR |=  (RCC_CFGR_PLLSRC_HSE_PREDIV        | /* configure PLL */
                 RCC_CFGR_PLLXTPRE_HSE_PREDIV_DIV1 |
                 RCC_CFGR_PLLMUL9                   );


  RCC->CR |= RCC_CR_PLLON;                           /* Enable PLL */
  while((RCC->CR & RCC_CR_PLLRDY) == 0) __NOP();     /* Wait till PLL is ready */

  RCC->CFGR &= ~(RCC_CFGR_SW);                       /* clear settings */
  RCC->CFGR |= RCC_CFGR_SW_PLL;                      /* Select PLL as system clock source */
  while ((RCC->CFGR & (uint32_t)RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL) __NOP();
}


/*----------------------------------------------------------------------------
 *      Main: Initialize and start RTX Kernel
 *---------------------------------------------------------------------------*/
void app_main (void *argument) {

  tid_phaseA = osThreadNew(phaseA, NULL, NULL);
  tid_phaseB = osThreadNew(phaseB, NULL, NULL);
  tid_phaseC = osThreadNew(phaseC, NULL, NULL);
  tid_phaseD = osThreadNew(phaseD, NULL, NULL);
  tid_clock  = osThreadNew(clock,  NULL, NULL);

  osThreadFlagsSet(tid_phaseA, 0x0001);          /* set signal to phaseA thread   */

  osDelay(osWaitForever);
  while(1);
}

int main (void) {

  // System Initialization
  SystemCoreClockConfigure();
  SystemCoreClockUpdate();
  LED_Initialize();
  // ...
  osKernelInitialize();                 // Initialize CMSIS-RTOS
  osThreadNew(app_main, NULL, NULL);    // Create application main thread
  if (osKernelGetState() == osKernelReady) {
    osKernelStart();                    // Start thread execution
  }

  while(1);
}


